@@ -82,6 +82,7 @@ gboolean capplet_set_boolean_toggle(CappletData *capp,
 {
     gboolean value = options_lookup_int_with_default(capp->options, name, dflt);
     capplet_set_toggle(capp, name, value);
+    return value;
 }
 
 void capplet_set_text_entry(CappletData *capp,
@@ -303,15 +304,12 @@ int main(int argc, char **argv)
     int n;
     const char *profile = NULL;
     const char *colour_scheme = NULL;
-    char *logo_filename;
     gboolean persist = FALSE;
     gboolean open_configlet = FALSE;
     gboolean dbus_ok;
 
     g_set_application_name(_("roxterm-config"));
 
-    global_options_init_appdir(argc, argv);
-
 #if ENABLE_NLS
     setlocale(LC_ALL, "");
     bindtextdomain(PACKAGE, global_options_appdir ?
@@ -320,6 +318,8 @@ int main(int argc, char **argv)
 #endif    
 
     gtk_init(&argc, &argv);
+    global_options_init(&argc, &argv, FALSE);
+    global_options_apply_dark_theme();
 
     resources_access_icon();
     gtk_window_set_default_icon_name("roxterm");
@@ -401,4 +401,4 @@ void capplet_dec_windows(void)
         gtk_main_quit();
 }
 
-/* vi:set sw=4 ts=4 noet cindent cino= */
+/* vi:set sw=4 ts=4 et cindent cino= */
