@@ -1,8 +1,8 @@
 #ifndef GETNAME_H
 #define GETNAME_H
 /*
-    roxterm - GTK+ 2.0 terminal emulator with tabs
-    Copyright (C) 2004 Tony Houghton <h@realh.co.uk>
+    roxterm - VTE/GTK terminal emulator with tabs
+    Copyright (C) 2004-2015 Tony Houghton <h@realh.co.uk>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,4 +35,4 @@ char *getname_run_dialog(GtkWindow *parent, const char *old_name,
 
 #endif /* GETNAME_H */
 
-/* vi:set sw=4 ts=4 noet cindent cino= */
+/* vi:set sw=4 ts=4 et cindent cino= */
