@@ -17,11 +17,11 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include "gdk/gdk.h"
 #ifndef DEFNS_H
 #include "defns.h"
 #endif
 
-#include <ctype.h>
 #include <errno.h>
 
 #include "dlg.h"
@@ -100,6 +100,10 @@ struct MultiWin {
     GdkGeometry geom_hints;
     GdkWindowHints geom_hint_mask;
     gboolean has_geometry;
+    gboolean show_clipboard_indicator;
+    int clipboard_flash_frame;
+    gulong clipboard_flash_tag;
+    GtkWidget *clipboard_indicator_button;
 };
 
 static double multi_win_zoom_factors[] = {
@@ -139,6 +143,7 @@ static MultiWinDeleteHandler multi_win_delete_handler;
 static MultiTabGetShowCloseButton multi_tab_get_show_close_button;
 static MultiTabGetNewTabAdjacent multi_tab_get_new_tab_adjacent;
 static MultiTabConnectMiscSignals multi_tab_connect_misc_signals;
+static MultiWinClipboardButtonHandler multi_win_clipboard_button_handler;
 
 static gboolean multi_win_notify_tab_removed(MultiWin *, MultiTab *);
 
@@ -252,8 +257,8 @@ void multi_tab_init(MultiTabFiller filler, MultiTabDestructor destructor,
     MultiWinDeleteHandler delete_handler,
     MultiTabGetShowCloseButton get_show_close_button,
     MultiTabGetNewTabAdjacent get_new_tab_adjacent,
-    MultiTabConnectMiscSignals connect_misc_signals
-    )
+    MultiTabConnectMiscSignals connect_misc_signals,
+    MultiWinClipboardButtonHandler clipboard_button_handler)
 {
     multi_tab_filler = filler;
     multi_tab_destructor = destructor;
@@ -269,6 +274,7 @@ void multi_tab_init(MultiTabFiller filler, MultiTabDestructor destructor,
     multi_tab_get_show_close_button = get_show_close_button;
     multi_tab_get_new_tab_adjacent = get_new_tab_adjacent;
     multi_tab_connect_misc_signals = connect_misc_signals;
+    multi_win_clipboard_button_handler = clipboard_button_handler;
 }
 
 // Doesn't connect client's signal handlers
@@ -623,14 +629,12 @@ static void multi_tab_remove_menutree_items(MultiWin * win, MultiTab * tab)
 static void renumber_tabs(MultiWin *win)
 {
     GList *link;
-    int n = 0;
 
     for (link = win->tabs; link; link = g_list_next(link))
     {
         MultiTab *tab = link->data;
 
         multi_tab_set_full_window_title(tab);
-        ++n;
     }
 }
 
@@ -1336,6 +1340,7 @@ static void multi_win_name_tab_action(MultiWin * win)
             multi_tab_set_window_title_template(tab, name);
             tab->title_template_locked = (name != NULL);
             /* Fall through to destroy */
+            G_GNUC_FALLTHROUGH;
         default:
             gtk_widget_destroy(dialog_w);
             break;
@@ -1344,6 +1349,7 @@ static void multi_win_name_tab_action(MultiWin * win)
     if (tab->postponed_free)
         multi_tab_free(tab);
 }
+
 static void multi_win_save_session_action(MultiWin * win)
 {
     GtkWidget *dialog_w = gtk_dialog_new_with_buttons(_("Save Session"),
@@ -1402,6 +1408,7 @@ static void multi_win_save_session_action(MultiWin * win)
                         filename, strerror(errno));
             }
             /* Fall through to destroy */
+            G_GNUC_FALLTHROUGH;
         default:
             gtk_widget_destroy(dialog_w);
             break;
@@ -1460,6 +1467,7 @@ static void multi_win_set_window_title_action(MultiWin * win)
             multi_win_set_title_template(win, title);
             win->title_template_locked = (title != NULL);
             /* Fall through to destroy */
+            G_GNUC_FALLTHROUGH;
         default:
             gtk_widget_destroy(dialog_w);
             break;
@@ -1874,6 +1882,22 @@ static void multi_win_size_allocate(GtkWidget *widget, GdkRectangle *alloc,
     }
 }
 
+static void multi_win_on_clipboard_button_clicked(MultiWin *win)
+{
+    MultiTab *tab = win ? win->current_tab : NULL;
+    gpointer data = tab ? tab->user_data : NULL;
+    if (data)
+    {
+        multi_win_clipboard_button_handler(data);
+    }
+    else
+    {
+        g_critical("Clipboard button clicked for NULL tab data");
+    }
+    if (win)
+        multi_win_hide_clipboard_indicator(win);
+}
+
 MultiWin *multi_win_new_blank(Options *shortcuts,
         int zoom_index,
         gboolean disable_menu_shortcuts, gboolean disable_tab_shortcuts,
@@ -2001,13 +2025,26 @@ MultiWin *multi_win_new_blank(Options *shortcuts,
         multi_win_set_show_tabs_menu_items(win, FALSE);
         multi_win_hide_tabs(win);
     }
+
+    GtkWidget *action_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+    win->clipboard_indicator_button =
+        multitab_close_button_new("edit-paste-symbolic");
+    g_signal_connect_swapped(win->clipboard_indicator_button, "clicked",
+            G_CALLBACK(multi_win_on_clipboard_button_clicked), win);
+    gtk_box_pack_start(GTK_BOX(action_box), win->clipboard_indicator_button,
+                       FALSE, FALSE, 0);
     win->add_tab_button = multitab_close_button_new("tab-new-symbolic");
     g_signal_connect_swapped(win->add_tab_button, "clicked",
             G_CALLBACK(multi_win_new_tab_action), win);
-    gtk_notebook_set_action_widget(notebook, win->add_tab_button, GTK_PACK_END);
+    gtk_box_pack_start(GTK_BOX(action_box), win->add_tab_button,
+                       FALSE, FALSE, 0);
+    gtk_notebook_set_action_widget(notebook, action_box, GTK_PACK_END);
+    gtk_widget_show(action_box);
+    gtk_widget_hide(win->clipboard_indicator_button);
     if (add_tab_button)
         gtk_widget_show_all(win->add_tab_button);
     win->show_add_tab_button = add_tab_button;
+
     gtk_box_pack_start(GTK_BOX(win->vbox), win->notebook, TRUE, TRUE, 0);
     g_signal_connect(win->notebook, "switch-page",
         G_CALLBACK(multi_win_page_switched), win);
@@ -2090,6 +2127,10 @@ static void multi_win_destructor(MultiWin *win, gboolean destroy_widgets)
     g_return_if_fail(win);
 
     win->ignore_tab_selections = TRUE;
+    if (win->clipboard_flash_tag != 0)
+    {
+        g_source_remove(win->clipboard_flash_tag);
+    }
     if (win->accel_group)
     {
         UNREF_LOG(g_object_unref(win->accel_group));
@@ -2511,6 +2552,11 @@ void multi_win_foreach_tab(MultiWin *win, MultiWinForEachTabFunc func,
     }
 }
 
+GList *multi_win_get_tabs(MultiWin *win)
+{
+    return win->tabs;
+}
+
 GtkPositionType multi_win_get_tab_pos(MultiWin *win)
 {
     return win->tab_pos;
@@ -2616,29 +2662,34 @@ static const char *parse_digits(const char *geom, int *num)
 
 /* Parse a signed integer, which might be preceded by a plus or minus.
  * Return NULL if invalid, or a pointer to where the parsing ended. */
-static const char *parse_signed(const char *geom, int *num)
+static const char *parse_signed(const char *geom, int *num, int *sign)
 {
     char c = *geom;
     if (c == '+' || c == '-')
         geom++;
     geom = parse_digits(geom, num);
+    if (geom && c == '+')
+    {
+        *sign = +1;
+    }
     if (geom && c == '-')
     {
         *num = (0 - *num);
+        *sign = -1;
     }
     return geom;
 }
 
-/* Parse a pair of signed integers, each must be preceded by a plus-sign.
+/* Parse a pair of signed integers, each must be preceded by a plus or minus.
  * Return NULL if invalid, or a pointer to where the parsing ended. */
-static const char *parse_geom_offsets(const char *g, int *x, int *y)
+static const char *parse_geom_offsets(const char *g, int *x, int *y, int *sign_x, int *sign_y)
 {
-    if (g && *g == '+')
+    if (g && ( *g == '+' || *g == '-' ) )
     {
-        g = parse_signed(g + 1, x);
-        if (g && *g == '+')
+        g = parse_signed(g, x, sign_x);
+        if (g && ( *g == '+' || *g == '-' ))
         {
-            return parse_signed(g + 1, y);
+            return parse_signed(g, y, sign_y);
         }
     }
     return NULL;
@@ -2647,7 +2698,7 @@ static const char *parse_geom_offsets(const char *g, int *x, int *y)
 /* Parse a geometry string WxH[+X+Y]. If pxy is non-null, then also
  * parse a position +X+Y. Return TRUE for WxH and set pxy for +X+Y. */
 gboolean multi_win_parse_geometry(const char *geom,
-        int *width, int *height, int *x, int *y, gboolean *pxy)
+        int *width, int *height, int *x, int *y, int *sign_x, int *sign_y, gboolean *pxy)
 {
     geom = parse_digits(geom, width);
     if (geom && strchr("xX", *geom))
@@ -2657,7 +2708,7 @@ gboolean multi_win_parse_geometry(const char *geom,
         {
             if (pxy)
             {
-                *pxy = (parse_geom_offsets(geom, x, y) != NULL);
+                *pxy = (parse_geom_offsets(geom, x, y, sign_x, sign_y) != NULL);
             }
             return TRUE;
         }
@@ -2744,10 +2795,12 @@ static gboolean multi_win_process_geometry(MultiWin *win,
 void multi_win_set_initial_geometry(MultiWin *win, const char *geom,
         MultiTab *tab)
 {
-    int columns, rows, x, y, width, height;
+    int columns, rows, x, y, sign_x, sign_y, width, height;
     gboolean xy;
+    GdkGravity pos_gravity;
+    int move_to_x, move_to_y;
 
-    if (multi_win_parse_geometry(geom, &columns, &rows, &x, &y, &xy))
+    if (multi_win_parse_geometry(geom, &columns, &rows, &x, &y, &sign_x, &sign_y, &xy))
     {
         //g_debug("multi_win_set_initial_geometry: %d columns, %d rows",
         //        columns, rows);
@@ -2762,10 +2815,43 @@ void multi_win_set_initial_geometry(MultiWin *win, const char *geom,
         {
             g_warning("Unable to determine geometry");
         }
-        /* Honor initial position on launch: it is indispensable and ubiquitous */
+        /* Honour initial position from --geometry on launch. This probably
+         * won't work on Wayland, but some people still find it useful on
+         * X11. The support for negative positioning and gravity is based on
+         * a patch by github user BMJ-pdx (issue #272). This could go wrong on
+         * multi monitor set-ups; GTK3 does not support it very well.
+         */
         if (xy)
         {
-            gtk_window_move(GTK_WINDOW(win->gtkwin), x, y);
+            /* See:
+             * https://developer.gnome.org/gtk3/stable/GtkWindow.html#gtk-window-move
+             * And example code in mate-terminal src/terminal-window.c.
+             */
+            GdkDisplay *display = gdk_display_get_default();
+            GdkMonitor *monitor = gdk_display_get_primary_monitor(display);
+            GdkRectangle monitor_geom;
+            gdk_monitor_get_geometry(monitor, &monitor_geom);
+            if (sign_x >= 0) {
+                move_to_x = x;
+                if (sign_y >= 0) {
+                    pos_gravity = GDK_GRAVITY_NORTH_WEST;
+                    move_to_y = y;
+                } else {
+                    pos_gravity = GDK_GRAVITY_SOUTH_WEST;
+                    move_to_y = monitor_geom.height - height + y;
+                }
+            } else {
+                move_to_x = monitor_geom.width - width + x;
+                if (sign_y >= 0) {
+                    pos_gravity = GDK_GRAVITY_NORTH_EAST;
+                    move_to_y = y;
+                } else {
+                    pos_gravity = GDK_GRAVITY_SOUTH_EAST;
+                    move_to_y = monitor_geom.height - height + y;
+                }
+            }
+            gtk_window_set_gravity(GTK_WINDOW(win->gtkwin), pos_gravity);
+            gtk_window_move(GTK_WINDOW(win->gtkwin), move_to_x, move_to_y);
         }
     }
 }
@@ -2798,6 +2884,50 @@ void multi_win_apply_new_geometry(MultiWin *win, int columns, int rows,
     }
 }
 
+/* Shows the indicator on odd frames, hides it on even */
+static gboolean multi_win_toggle_clipboard_indicator(MultiWin *win)
+{
+    // Use opacity instead of show/hide in case the tab bar is full and
+    // the tabs would grow and shrink if the visibility were toggled.
+    gtk_widget_set_opacity(win->clipboard_indicator_button,
+                           (win->clipboard_flash_frame % 2) ? 1.0 : 0.0);
+    if ((!--win->clipboard_flash_frame && win->show_clipboard_indicator) ||
+        win->clipboard_flash_frame < 0)
+    {
+        if (!win->show_clipboard_indicator)
+        {
+            gtk_widget_hide(win->clipboard_indicator_button);
+        }
+        win->clipboard_flash_tag = 0;
+        return FALSE;
+    }
+    return TRUE;
+}
+
+void multi_win_flash_clipboard_indicator(MultiWin *win, gboolean show)
+{
+    gboolean already_showing = win->show_clipboard_indicator;
+    win->show_clipboard_indicator = show;
+    if (!win->clipboard_flash_tag)
+    {
+        win->clipboard_flash_frame = already_showing ? 5 : 4;
+        gtk_widget_show_all(win->clipboard_indicator_button);
+        win->clipboard_flash_tag =
+            g_timeout_add(200,
+                          G_SOURCE_FUNC(multi_win_toggle_clipboard_indicator),
+                          win);
+    }
+}
+
+void multi_win_hide_clipboard_indicator(MultiWin *win)
+{
+    win->show_clipboard_indicator = FALSE;
+    if (!win->clipboard_flash_tag)
+    {
+        gtk_widget_hide(win->clipboard_indicator_button);
+    }
+}
+
 /* This is maintained in order of most recently focused */
 GList *multi_win_all = NULL;
 
