@@ -1,8 +1,8 @@
 #ifndef SHORTCUTS_H
 #define SHORTCUTS_H
 /*
-    roxterm - GTK+ 2.0 terminal emulator with tabs
-    Copyright (C) 2004 Tony Houghton <h@realh.co.uk>
+    roxterm - VTE/GTK terminal emulator with tabs
+    Copyright (C) 2004-2015 Tony Houghton <h@realh.co.uk>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,9 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include "defns.h"
 
+#ifndef ROXTERM_CAPPLET
 #include "options.h"
 
 /* Call once to connect signal to save shortcuts when changed by user; or you
@@ -27,13 +29,33 @@
 void shortcuts_init(void);
 
 /* Loads named keyboard shortcuts scheme, adding accelerators to the global
- * GtkAccelMap. If file not found or NULL, tries to load Default */
-Options *shortcuts_open(const char *scheme_name);
+ * GtkAccelMap. If file not found or NULL, tries to load Default. If reload
+ * is TRUE it forces the shortcuts to be reloaded from the file even if a
+ * scheme of the same name is already loaded.
+ */
+Options *shortcuts_open(const char *scheme_name, gboolean reload);
 
 void shortcuts_unref(Options *scheme);
 
+#if GTK_CHECK_VERSION(3, 10, 0)
+inline static void shortcuts_enable_signal_handler(gboolean enable)
+{
+    (void) enable;
+}
+#else
 void shortcuts_enable_signal_handler(gboolean enable);
+#endif
+
+gboolean shortcuts_key_is_shortcut(Options *shortcuts,
+        guint key, GdkModifierType modifiers);
+
+const char *shortcuts_get_index_str(Options *shortcuts);
+
+#endif /* ROXTERM_CAPPLET */
+
+/* window = parent window for dialogs in case of error etc */
+void shortcuts_edit(GtkWindow *window, const char *name);
 
 #endif /* SHORTCUTS_H */
 
-/* vi:set sw=4 ts=4 noet cindent cino= */
+/* vi:set sw=4 ts=4 et cindent cino= */
